/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file GeneralSettingsFileManager.h
   
  \brief Class responsible for managing values added or removed to the file referenced by QSettings.
    QSettings (Doc): Users normally expect an application to remember its settings (window sizes and positions, options, etc.) across sessions. 
    This information is often stored in the system registry on Windows, and in XML preferences files on Mac OS X. 
    On Unix systems, in the absence of a standard, many applications (including the KDE applications) use INI text files.
    Scope: user.
    Default settings: "TerraLib"(Organization) and "TerraPrint"(Application)

  \ingroup layout
*/

#ifndef __TERRALIB_LAYOUT_INTERNAL_GENERAL_SETTINGS_FILE_MANAGER_H
#define __TERRALIB_LAYOUT_INTERNAL_GENERAL_SETTINGS_FILE_MANAGER_H

// TerraLib
#include "../../core/Config.h"

#include <QString>
#include <QMap>
#include <QMap>

class QVariant;

namespace te
{
  namespace layout
  {
    class SharedIOAccess;
    /*!
      \brief Class responsible for managing values added or removed to the file referenced by QSettings.
      QSettings (Doc): Users normally expect an application to remember its settings (window sizes and positions, options, etc.) across sessions. 
      This information is often stored in the system registry on Windows, and in XML preferences files on Mac OS X. 
      On Unix systems, in the absence of a standard, many applications (including the KDE applications) use INI text files.
      Scope: user.
      Default settings: "TerraLib"(Organization) and "TerraPrint"(Application)
    
    \ingroup layout
    */
    class TELAYOUTEXPORT GeneralSettingsFileManager
    {
      public:

        /*!
          \brief Constructor

          \param sharedIOAccess manages disk access requests
          \param organization company name
          \param application application name
        */
        GeneralSettingsFileManager(SharedIOAccess* sharedIOAccess, const QString& organization = "", const QString& application = "");

        /*!
          \brief Destructor
        */
        virtual ~GeneralSettingsFileManager();

        /*!
          \brief Get a setting (QSettings) from the key.

          \param key key to search
          \return value found
        */
        QVariant getSetting(const QString& key);

        /*!
          \brief Add a setting in the file that QSettings points to.

          \param key setting key
          \param variant setting value
          \return if true it was written, false otherwise
        */
        bool writeSetting(const QString& key, const QVariant& variant);

        /*!
          \brief Add a group settings to the file that QSettings points to.

          \param group setting group
          \param key setting key
          \param variant setting value
          \return if true it was written, false otherwise
        */
        bool writeSetting(const QString& group, const QString& key, const QVariant& variant);

        /*!
          \brief Add multiple group settings to the file that QSettings points to.

          \param group setting group
          \param listSettings setting list
          \return if true it was written, false otherwise
        */
        bool writeSettings(const QString& group, const QMap<QString, QVariant>& listSettings);

        /*!
          \brief Get settings from a group.

          \param group group to search
          \return map with settings key/value (alias/prefix)
        */
        QMap<QString, QVariant> getGroupSettings(const QString& group);
        
        /*!
          \brief Remove settings (QSettings) from the key.

          \param key key to search
          \return if true was removed, false otherwise
        */
        bool removeSetting(const QString& key);

        /*!
          \brief Remove settings (QSettings) from a group.

          \param group group to search
          \param key key to search
          \return if true was removed, false otherwise
        */
        bool removeSetting(const QString& group, const QString& key);

        /*!
          \brief Default group for Image Gallery settings.

          \return default image gallery group name
        */
        QString getDefaultImageGalleryGroup();

        /*!
          \brief Default TerraPrint alias.

          \return default layout alias name
        */
        QString getDefaultLayoutAlias();

        /*!
          \brief Default TerraPrint prefix.

          \return default layout prefix path
        */
        QString getDefaultLayoutPrefix();

        /*!
          \brief Add a pair key/value to a internal list of prefixes. 
            The prefix has to follow the file path form of the Qt Resource System ":/".

          \param alias will be the key
          \param prefix will be the value
          \return if added true, false otherwise
        */
        bool addPrefix(const QString& alias, const QString& prefix);

        /*!
          \brief Remove a pair key/value of a internal list of prefixes.
         
          \param alias key to search
          \return if removed true, false otherwise
        */
        bool removePrefix(const QString& alias);

        /*!
          \brief Get all the registered prefixes. Pair key (alias) and value (prefix).

          \return map with settings key/value (alias/prefix) 
        */
        QMap<QString, QString> getPrefixList();

      protected:

        SharedIOAccess*            m_sharedIOAccess; //<! Manages disk access requests
        QString                    m_organization; //<! Company name
        QString                    m_application; //<! Application name
        QString                    m_defaultImageGalleryGroup; //<! Default image gallery group name 
        QMap<QString, QString>     m_prefixList; //!< Map with settings key/value (alias/prefix) 
        QString                    m_defaultLayoutAlias; //<! Default image gallery alias name 
        QString                    m_defaultLayoutPrefix; //<! Default image gallery prefix path 
    };
  }
}

#endif 
