/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file MoveItems.h
   
  \brief Class to move items. Items are moved one millimeter (default) to right, left, up or down. 
    Usually the methods of this class are used when arrow keys are pressed by the user.

  \ingroup layout
*/

#ifndef __TERRALIB_LAYOUT_INTERNAL_MOVE_ITEMS_H
#define __TERRALIB_LAYOUT_INTERNAL_MOVE_ITEMS_H

// TerraLib
#include "../../core/Config.h"
#include "../../core/PaperConfig.h"

// STL
#include <map>

// Qt

class QRectF;
class QPointF;
class QGraphicsItem;

namespace te
{
  namespace layout
  {
    class Scene;
    class Property;
    /*!
      \brief Class to move items. Items are moved one millimeter (default) to right, left, up or down. 
        Usually the methods of this class are used when arrow keys are pressed by the user.
    
    \ingroup layout
    */
    class TELAYOUTEXPORT MoveItems
    {
      public:

        /*!
          \brief Constructor

          \param scene graphics view scene
        */
        MoveItems(Scene* scene);

        /*!
          \brief Destructor
        */
        virtual ~MoveItems();
        
        /*!
          \brief Move selected items 1mm to the left.
        */
        virtual void moveToLeft();

        /*!
          \brief Move selected items 1mm to the right.
        */
        virtual void moveToRight();

        /*!
          \brief Move selected items 1mm up.
        */
        virtual void moveToUp();

        /*!
          \brief Move selected items 1mm down.
        */
        virtual void moveToDown();

        /*!
          \brief Increased or decreased value to move item.

          \param value in millimeters
        */
        void setMoveValue(double newValue);
                
      protected:

        /*!
          \brief Updates properties and adds changes to Undo/Redo.

          \param items items that will be updated
          \return if true changes done, false otherwise
        */
        virtual bool addChangePropertiesToUndoRedoStack(const std::map<QGraphicsItem*, QPointF>& items);

        /*!
          \brief Move items on the X axis.

          \param moveValue value to move item (millimeter)
        */
        virtual void moveAxisX(double moveValue);

        /*!
          \brief Move items on the Y axis.

          \param moveValue value to move item (millimeter)
        */
        virtual void moveAxisY(double moveValue);
                
        Scene*          m_scene; //!< Graphics view scene
        double          m_moveValueMM; //!< Increased or decreased value to move item (millimeters)
    };
  }
}

#endif 
