/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file ImageItem.h
   
   \brief Class that represents a image item. 

  \ingroup layout
*/

#ifndef __TERRALIB_LAYOUT_INTERNAL_IMAGE_ITEM_H 
#define __TERRALIB_LAYOUT_INTERNAL_IMAGE_ITEM_H

// TerraLib
#include "../../core/Config.h"
#ifndef Q_MOC_RUN
#include "AbstractItem.h"
#endif

// Qt
#include <QImage>

namespace te
{
  namespace layout
  {
    class AbstractItemController;
    class AbstractItemModel;
    /*!
    \brief Class that represents a image item. 

      \ingroup layout

      \sa te::layout::AbstractItem
    */
    class TELAYOUTEXPORT ImageItem : public AbstractItem
    {
      public:

        /*!
          \brief Constructor

          \param itemInputProxy proxy for external item settings
        */ 
        ImageItem(te::layout::ItemInputProxy* itemInputProxy);

        /*!
          \brief Destructor
         */
        virtual ~ImageItem();

        /*!
        \brief Gets the file name of the image
        */
        virtual const std::string& getFileName() const;

        /*!
        \brief Sets the file name of the image
        */
        virtual void setFileName(const std::string& fileName);

        /*!
          \brief Calculates the best aspect ratio between width and height and updates properties.
        */
        virtual void adjustSize();

      protected:

        /*!
          \brief Create the model using Lazy Initialization (initializes a variable or attribute on it's first access).
          
          \return model
        */
        virtual AbstractItemModel* createModel() const;

        /*!
          \brief Create the controller using Lazy Initialization (initializes a variable or attribute on it's first access).

          \return controller
        */
        virtual AbstractItemController* createController() const;

        /*!
          \brief For any specific drawing, the item must reimplement this function.

          \param painter current active painter for drawing
          \param option describe the parameters needed to draw a QGraphicsItem
          \param widget argument is optional. If provided, it points to the widget that is being painted on
        */
        virtual void drawItem (QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget = 0);

        /*!
          \brief Svg drawing.

          \param painter current active painter for drawing
        */
        virtual void drawSVG(QPainter * painter);

        /*!
          \brief Image drawing.

          \param painter current active painter for drawing
        */
        virtual void drawImage(QPainter * painter);

      protected:

        QImage      m_image; //!< Represents the loaded image
        std::string m_fileName; //!< The file name of the loaded image
    };
  }
}

#endif
