/*  Copyright (C) 2001-2014 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file ImageGalleryDialogController.h
   
  \brief Image gallery window controller.

  \ingroup layout
*/

#ifndef __TERRALIB_LAYOUT_INTERNAL_IMAGE_GALLERY_DIALOG_CONTROLLER_H 
#define __TERRALIB_LAYOUT_INTERNAL_IMAGE_GALLERY_DIALOG_CONTROLLER_H

// TerraLib
#include "../../core/pattern/mvc/AbstractOutsideController.h"
#include "../../core/Config.h"
#include "../../core/pattern/proxy/AbstractProxyProject.h"

// Qt
#include <QMap>
#include <QString>

class QVariant;

namespace te
{
  namespace layout
  {
    class AbstractOutsideModel;
    class AbstractProxyProject;
    class Property;
    class Scene;
    /*!
      \brief Image gallery window controller.

      \ingroup layout
    */
    class TELAYOUTEXPORT ImageGalleryDialogController : public AbstractOutsideController
    {
      public:

        /*!
          \brief Constructor

          \param scene graphics view scene
          \param proxy provide a surrogate or placeholder for te::qt::af::Project to control access to it
          \param o "Model" part of MVC component
        */
        ImageGalleryDialogController(Scene * scene, AbstractProxyProject * proxy, AbstractOutsideModel* o);

        /*!
          \brief Destructor
        */
        virtual ~ImageGalleryDialogController();

        /*!
          \brief Get a property by name

          \param name property name to search
          \return property found
        */
        virtual Property getProperty(const std::string& name);

        /*!
          \brief Get a setting (QSettings) from the key.

          \param key key to search
          \return value found
        */
        QVariant getSetting(const QString& key);
        
        /*!
          \brief Add multiple group settings to the file that QSettings points to.

          \param listSettings setting list
          \return if true it was written, false otherwise
        */
        bool writeSettingsToGroup(const QMap<QString, QVariant> listSettings);
        
        /*!
          \brief Add a group settings to the file that QSettings points to.

          \param key setting key
          \param variant setting value
          \return if true it was written, false otherwise
        */
        bool writeSettingToGroup(const QString& key, const QVariant& value);

        /*!
          \brief Get settings from a group.

          \return map with settings key/value (alias/prefix)
        */
        QMap<QString, QVariant> getGroupSettings();

        /*!
          \brief Remove settings (QSettings) from the key.

          \param key key to search
          \return if true was removed, false otherwise
        */
        bool removeSetting(const QString& key);

        /*!
          \brief Default TerraPrint alias.

          \return default layout alias name
        */
        QString getDefaultLayoutAlias();

        /*!
          \brief Default TerraPrint prefix.

          \return default layout prefix path
        */
        QString getDefaultLayoutPrefix();

        /*!
          \brief Get all the registered prefixes. Pair key (alias) and value (prefix).

          \return map with settings key/value (alias/prefix)
        */
        QMap<QString, QString> getPrefixList();
                
      private:
        Scene*                  m_scene; //!< Graphics view scene
        AbstractProxyProject*   m_proxy; //!< Provide a surrogate or placeholder for te::qt::af::Project to control access to it
    };
  }
}

#endif
