/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/layout/qt/outside/ImageGalleryDialogOutside.h

  \brief Image gallery window. Manages resource paths and image folders.
*/

#ifndef __TERRALIB_LAYOUT_INTERNAL_IMAGE_GALLERY_OUTSIDE_H
#define __TERRALIB_LAYOUT_INTERNAL_IMAGE_GALLERY_OUTSIDE_H

// TerraLib
#include "terralib/qt/widgets/utils/DoubleListWidget.h"
#include "../../core/Config.h"
#ifndef Q_MOC_RUN
#include "../../core/pattern/mvc/AbstractOutsideView.h"
#endif

// STL
#include <memory>
#include <string>
#include <vector>
#include <list>

// Qt
#include <QDialog>
#include <QList>
#include <QSize>

class QListWidgetItem;
class QTreeWidgetItem;
class QString;
class QStringList;
class QListWidgetItem;
class QKeyEvent;

namespace Ui { class ImageGallery; }

namespace te
{
  namespace layout
  {
    class AbstractOutsideController;
    /*!
      \brief Image gallery window. Manages resource paths and image folders.

      \ingroup layout
    */
    class TELAYOUTEXPORT ImageGalleryDialogOutside : public QDialog, public AbstractOutsideView
    {
      Q_OBJECT

      public:

        /*!
          \brief Constructor

          \param controller "Controller" part of MVC component
          \param parent parent widget
        */
        ImageGalleryDialogOutside(AbstractOutsideController* controller, QWidget* parent);

        /*!
          \brief Destructor
        */
        ~ImageGalleryDialogOutside();

        /*!
          \brief Load layers in double widget.
        */
        virtual void init();
        
        /*!
          \brief Reimplement from AbstractOutsideView.
        */
        virtual void setPosition( const double& x, const double& y );

        /*!
          \brief Reimplement from AbstractOutsideView.
        */
        virtual te::gm::Coord2D getPosition();

        /*!
          \brief Set current property.

          \param prop current property
        */
        virtual void setPathsProperty( Property prop );

      signals:

        /*!
          \brief Sends signal that a property needs to be updated on current items.

          \param prop property to be updated
        */
        void updateProperty(Property prop);

        /*!
          \brief Sends signal that multiple properties need to be updated on current items.

          \param props properties to update
        */
        void updateProperties(std::vector<Property> props);
        
      protected slots:

        /*!
          \brief Method connected to apply button click.
        */
        virtual void on_m_pbtnApply_clicked();

        /*!
          \brief Method connected to OK button click.
        */
        virtual void on_m_pbtnOK_clicked();

        /*!
          \brief Method connected to close button click.
        */
        virtual void on_m_pbtnClose_clicked();

        /*!
          \brief Method connected to add button click.
        */
        virtual void on_m_pbtnAdd_clicked();

        /*!
          \brief Method connected to remove button click.
        */
        virtual void on_m_pbtnRemove_clicked();
        
        /*!
          \brief Tree-connected method when an item is clicked

          \param item tree item clicked
          \param column tree column clicked
        */
        virtual void on_m_treeGroups_itemClicked(QTreeWidgetItem* item, int column);
        
        /*!
          \brief Method connected to list of images when index of current item changes.

          \param current current item of the list
          \param previous previous item of the list
        */
        virtual void on_m_listImages_currentItemChanged(QListWidgetItem * current, QListWidgetItem * previous);

        /*!
          \brief Method connected to the size combo when the index of the current item changes.

          \param index current index
        */
        virtual void on_m_cmbIconSizes_currentIndexChanged(int index);

        /*!
          \brief Filters the images viewed by the name that has parts in the text.

          \param text text to filter display images by name
        */
        virtual void on_m_lneFilter_textChanged(const QString & text);

      protected:

        /*!
          \brief Reimplemented to receive key press events for this item.

          \param e key press event object
        */
        virtual void keyPressEvent(QKeyEvent* e);

        /*!
          \brief Create tree items. Tree has 2 main subdivisions: library and folder.
        */
        virtual void loadTree();

        /*!
          \brief Load the settings in the tree.
        */
        virtual void loadSettings();

        /*!
          \brief Load image details.
        */
        virtual void loadTable(QListWidgetItem* item);

        /*!
          \brief Load images in the list.

          \param 
        */
        virtual void loadImages(const QList<QString>& listDir);

        /*!
          \brief Load image sizes in the combobox.
        */
        virtual void loadComboIconSizes();

        /*!
          \brief Load the current image path in the line edit.
        */
        virtual void loadCurrentPath();

        /*!
          \brief Load in the tree all resource by alias/prefix.
        */
        virtual void loadPrefixList();
        
        /*!
          \brief Depth search in all levels in a directory.

          \param topDir search directory
          \return list with all sub directories
        */
        virtual QList<QString> getAllSubDirectories(const QString& topDir);

        /*!
         \brief Create a list item with a Svg pixmap.

         \param index insertion order
         \param file complete path to load file
         \param fileName file name
        */
        virtual void addSvgToList(int index, const QString& file, const QString& fileName);

        /*!
          \brief Create a list item with a Image pixmap.

          \param index insertion order
          \param file complete path to load file
          \param fileName file name
        */
        virtual void addImageToList(int index, const QString& file, const QString& fileName);

        /*!
          \brief Get a list of formats that QImageReader can read.

          \return list of formats that QImageReader can read
        */
        virtual QStringList getSupportedFormats();

        /*!
          \brief Add a new directory to settings.

          \return if true directory was added, false otherwise
        */
        virtual bool add();
        
        /*!
          \brief Remove the current directory from settings.

          \return if true directory was removed, false otherwise.
        */
        virtual bool remove();
        
        /*!
          \brief Add subdirectories to the tree.

          \param root item that will have subitems
          \param dir directory that will be read
        */
        virtual void addSubDirsToTree(QTreeWidgetItem* root, const QString& dir);

        /*!
          \brief Create a new tree item.

          \param parent item that will have sub items
          \param label visible text 
          \param path complete image/svg path
          \param hasToolTip if true the item will have tooltip, false otherwise
        */
        virtual QTreeWidgetItem* createTreeItem(QTreeWidgetItem* parent, const QString& label, const QString& path, bool hasToolTip = false);

        /*!
          \brief Create a new list item.

          \param index insertion order
          \param ico icon
          \param fileName visible text 
          \param completePath complete image/svg path
        */
        QListWidgetItem* createListItem(int index, const QIcon& ico, const QString& fileName, const QString& completePath);

        /*!
          \brief Get all paths from items tree.

          \param root search root item
          \return all paths
        */
        virtual QList<QString> getAllPathsFromTree(QTreeWidgetItem* root);

        /*!
          \brief Refresh table with details of the current tree item.
        */
        virtual void refreshCurrentTreeItem();

        /*!
          \brief Check if the item is the basic library.

          \param item tree item for check
          \return if true is basic library, false otherwise
        */
        virtual bool isBasicLibrary(QTreeWidgetItem* item);

        /*!
          \brief Check if the path is the basic library.

          \param path path for check
          \return if true is basic library, false otherwise
        */
        virtual bool isBasicLibrary(const QString& path);

        /*!
          \brief New current tree item.

          \param item new current tree item
        */
        virtual void itemChanged(QTreeWidgetItem* item);

        /*!
          \brief Update all required properties for the current graph item.

          \return if true It has been updated, false otherwise
        */
        virtual bool updateCurrentItemProperty();

        /*!
          \brief Returns the correct separator type for the specified path. Ex.: "/" or "\".

          \param path complete path
          \return correct separator
        */
        QString getSeparator(const QString& path);

        /*!
          \brief Create the basic tree items.
        */
        virtual void createBasicTreeItems();
        
      private:

        std::auto_ptr<Ui::ImageGallery>                   m_ui; //!< auto pointer to user interface
        Property                                          m_property; //!< current property
        QString                                           m_currentFile; //!< current image/svg file path
        QSize                                             m_currentIconSize; //!< current icons size
        QTreeWidgetItem*                                  m_library; //!< library tree item (Tree has 2 main subdivisions: library and folder)
        QTreeWidgetItem*                                  m_folder; //!< folder tree item (Tree has 2 main subdivisions: library and folder)
    };
  }    
}     

#endif

