#
#  Copyright (C) 2008-2014 National Institute For Space Research (INPE) - Brazil.
#
#  This file is part of the TerraLib - a Framework for building GIS enabled applications.
#
#  TerraLib is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TerraLib is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TerraLib. See COPYING. If not, write to
#  TerraLib Team at <terralib-team@terralib.org>.
#
#
#  Description: Auxiliary macros and functions.
#
#  Author: Gilberto Ribeiro de Queiroz <gribeiro@dpi.inpe.br>
#          Juan Carlos P. Garrido <juan@dpi.inpe.br>
#          Frederico Augusto T. Bede <frederico.bede@funcate.org.br>
#

MACRO(TeInstallPlugins plugins location)

  set(_files)

  foreach(plugin ${plugins})
    get_target_property(_loc ${plugin} LOCATION)
    if(EXISTS ${_loc})
        list(APPEND _files ${_loc})
    endif()
  endforeach()

  install(FILES ${_files}
           DESTINATION "${TERRALIB_BASE_DESTINATION_DIR}qtplugins/${location}"
           CONFIGURATIONS Release
           COMPONENT runtime)

ENDMACRO(TeInstallPlugins)


MACRO(TeInstallQt5Plugins)
  find_package(Qt5 COMPONENTS Sql Svg PrintSupport)

# Installing image plugins
  set(_plugins Qt5::QGifPlugin Qt5::QICOPlugin Qt5::QJpegPlugin Qt5::QMngPlugin Qt5::QTiffPlugin Qt5::QSvgPlugin)
  TeInstallPlugins("${_plugins}" "imageformats")

# Installing svg plugins
  set(_plugins Qt5::QSvgIconPlugin)
  TeInstallPlugins("${_plugins}" "iconengines")

# Installing sql plugins
  set(_plugins Qt5::QSQLiteDriverPlugin)
  TeInstallPlugins("${_plugins}" "sqldrivers")

# Installing printer support plugin
  if(WIN32)
    set(_plugins Qt5::QWindowsPrinterSupportPlugin)
    TeInstallPlugins("${_plugins}" "printsupport")
  endif()

# Installing platform plugins
  if(WIN32)
    set(_plugins Qt5::QWindowsIntegrationPlugin Qt5::QMinimalIntegrationPlugin)
    TeInstallPlugins("${_plugins}" "platforms")
  elseif(APPLE)
    set(_plugins Qt5::QCocoaIntegrationPlugin Qt5::QMinimalIntegrationPlugin)
    TeInstallPlugins("${_plugins}" "platforms")
  endif()

ENDMACRO(TeInstallQt5Plugins)

#
# Macro installQtPlugins
#
# Description: Installs the required Qt plugins.
#
# param plugs List of the names of plugins to be installed.
#
MACRO(TeInstallQtPlugins plgs)

  if ("${QT_PLUGINS_DIR}" STREQUAL "")
    set (QT_PLUGINS_DIR "${Qt5_DIR}/../../../plugins")
  endif()

  set (_regex_exp "")

  set( _first TRUE)

  foreach(plg ${plgs})
    if(NOT _first)
      set (_regex_exp ${_regex_exp}|${plg})
    else()
      set (_regex_exp ${plg})
      set (_first FALSE)
    endif()
  endforeach()

  set (_regex_exp "(${_regex_exp})?(${CMAKE_SHARED_LIBRARY_SUFFIX})$")

  set (_dest "${TERRALIB_BASE_DESTINATION_DIR}qtplugins")

if(QT4_FOUND)
  set (_plugin_dirs "imageformats;iconengines;sqldrivers")
else()
    set (_plugin_dirs "imageformats;iconengines;sqldrivers;platforms")
endif()

  foreach(_plugin_dir ${_plugin_dirs})

    install (DIRECTORY ${QT_PLUGINS_DIR}/${_plugin_dir}
             DESTINATION ${_dest}  COMPONENT runtime
             FILES_MATCHING REGEX "${_regex_exp}")
  endforeach()

ENDMACRO(TeInstallQtPlugins)

#
# Macro TeInstallTerraLibPlugins
#
# Description: Installs the required TerraLib plugins. (Used by outter projects.)
#
# param plugins List of the names of plugins to be installed.
#
# param location Path where the plugins must be installed.
#
MACRO(TeInstallTerraLibPlugins plugins location)
  set(_files "")

  foreach(plugin ${plugins})

    get_target_property(_loc ${plugin} LOCATION_RELEASE)

    list(APPEND _files "${_loc}")

    get_filename_component(_lib_dir ${_loc} DIRECTORY)
    
    set(_sym_loc "${_lib_dir}/${CMAKE_SHARED_LIBRARY_PREFIX}${plugin}${CMAKE_SHARED_LIBRARY_SUFFIX}")

    list(APPEND _files "${_sym_loc}")
    
  endforeach()

  install (FILES ${_files}
    DESTINATION ${location}
    CONFIGURATIONS Release
    COMPONENT runtime
  )

  set(_filesd "")

  foreach(plugin ${plugins})

    get_target_property(_loc ${plugin} LOCATION_DEBUG)

    list(APPEND _filesd "${_loc}")

    get_filename_component(_lib_dir ${_loc} DIRECTORY)
    
    set(_sym_loc "${_lib_dir}/${CMAKE_SHARED_LIBRARY_PREFIX}${plugin}${CMAKE_SHARED_LIBRARY_SUFFIX}")

    list(APPEND _filesd "${_sym_loc}")
    
  endforeach()

  install (FILES ${_files}
    DESTINATION ${location}
    CONFIGURATIONS Debug
    COMPONENT runtime
  )
  
ENDMACRO(TeInstallTerraLibPlugins)

MACRO(GetTerraLibModules _modules)
  set(allModules "${TERRALIB_MODULES}" "${TERRALIB_PLUGINS}")

  foreach(module ${allModules})
    set (_mod_name terralib_mod_${module})

#    string(FIND ${_mod_name} binding_python _py_ok)

#    if(${_py_ok} GREATER 0)
#      set(_mod_name _${_mod_name})
#    endif()

    list(APPEND ${_modules} ${_mod_name})
  endforeach()
ENDMACRO(GetTerraLibModules)


#
# Macro: TERRALIB_ADD_PYTHON_BINDING
#
# Description: Register a python binding for a terralib module
#
# param _module terralib module to be used in the binding
#
# param _source SWIG source file
MACRO(TERRALIB_ADD_PYTHON_BINDING _module _source)

file(GLOB TERRALIB_BINDINGS_PYTHON_SRC_FILES ${_source})

source_group("Source Files" FILES ${TERRALIB_BINDINGS_PYTHON_SRC_FILES})

string(REPLACE "terralib_mod_" "" py_module ${_module})


set_source_files_properties(${TERRALIB_BINDINGS_PYTHON_SRC_FILES}
                            PROPERTIES CPLUSPLUS ON CMAKE_SWIG_FLAGS "-includeall")

SWIG_ADD_MODULE(${py_module} python ${TERRALIB_BINDINGS_PYTHON_SRC_FILES})

SWIG_LINK_LIBRARIES(${py_module} "${_module}" ${PYTHON_LIBRARIES})

set_target_properties("_${py_module}"
                      PROPERTIES
                      LIBRARY_OUTPUT_DIRECTORY ${CMAKE_SWIG_OUTDIR}
                      LIBRARY_OUTPUT_DIRECTORY_RELEASE ${CMAKE_SWIG_OUTDIR}
                      LIBRARY_OUTPUT_DIRECTORY_DEBUG ${CMAKE_SWIG_OUTDIR}
                      DEBUG_POSTFIX ""
                      PREFIX "")

if(TERRALIB_PYTHON_BINDINGS)
  set(TERRALIB_PYTHON_BINDINGS "${TERRALIB_PYTHON_BINDINGS}, ${py_module}")
else()
  set(TERRALIB_PYTHON_BINDINGS "${py_module}")
endif()

ENDMACRO(TERRALIB_ADD_PYTHON_BINDING)

#
# Macro: TERRALIB_ADD_C_WRAPPER
#
# Description: Register a C wrapper for a terralib module
#
# param _module terralib module to be used in the wrapper
#
# param _dir C source dir
MACRO(TERRALIB_ADD_C_BINDING _module _dir)

file(GLOB HDR_FILES ${_dir}/*.h)
file(GLOB SRC_FILES ${_dir}/*.cpp)

string(REPLACE "terralib_mod_" "" folder ${_module})

source_group("Header Files\\${folder}" FILES ${HDR_FILES})
source_group("Source Files\\${folder}" FILES ${SRC_FILES})

set(c_module "${_module}_c")


add_library(${c_module} SHARED ${HDR_FILES}
                               ${SRC_FILES})

set_target_properties(${c_module}
                     PROPERTIES VERSION ${TERRALIB_VERSION_MAJOR}.${TERRALIB_VERSION_MINOR}
                                SOVERSION ${TERRALIB_VERSION_MAJOR}.${TERRALIB_VERSION_MINOR})
target_link_libraries(${c_module} ${_module})


ENDMACRO(TERRALIB_ADD_C_BINDING)

#
# Macro: TERRALIB_REGISTER_TRANSLATION
#
# Description: Register the ".mo" file for a given ".po" file
#
# param proj_name The module to be translated. This will be your text domain.
#
# param locale The locale of the ".po" file
#
# param po_dir Where the ".po" file is located. The ".po" file named must be: "proj_name"_"locale".po
#
# param mo_dir Output directory for the ".mo" file
#

MACRO(TERRALIB_REGISTER_TRANSLATION proj_name locale po_dir mo_dir)
  if(XGETTEXT_FOUND)
    if(TERRALIB_TRANSLATOR_ENABLED)
      set(_po_file "${po_dir}/${proj_name}_${locale}.po")
      set(_mo_file "${mo_dir}/${locale}/LC_MESSAGES/${proj_name}.mo")
      if(EXISTS ${po_dir}/${proj_name}_${locale}.po)
        add_custom_command(
          TARGET ${proj_name}
          PRE_BUILD
          COMMAND ${CMAKE_COMMAND} -E make_directory ${mo_dir}/${locale}/LC_MESSAGES/
          COMMAND ${GETTEXT_MSGFMT_EXECUTABLE} -o ${_mo_file} ${_po_file}
          COMMENT "Generating translation binary file for ${proj_name}...")
      endif()
    endif(TERRALIB_TRANSLATOR_ENABLED)
  endif(XGETTEXT_FOUND)
ENDMACRO(TERRALIB_REGISTER_TRANSLATION)

MACRO(TERRALIB_CREATE_TRANSLATION proj_name keyword_s keyword_p locale directory po_dir)
  if(XGETTEXT_FOUND)
    if(TERRALIB_TRANSLATOR_ENABLED)
      get_filename_component(_absDir ${directory} ABSOLUTE)
      set(_pot_file ${CMAKE_CURRENT_BINARY_DIR}/${proj_name}.pot)
      set(_po_file "${po_dir}/${proj_name}_${locale}.po")
      file(GLOB_RECURSE _srcs RELATIVE ${_absDir} ${_absDir}/*.cpp)
      set(encoding "UTF-8")
      if(NOT EXISTS "${po_dir}/${proj_name}_${locale}.po")
        configure_file(${po_dir}/template_translation.po.in
                       ${po_dir}/${proj_name}_${locale}.po @ONLY)
      endif()
      add_custom_command(
        TARGET ${proj_name}
        PRE_BUILD
        COMMAND ${XGETTEXT_EXECUTABLE} --from-code=${encoding} --no-wrap --c++ --keyword=${keyword_s} --keyword=${keyword_p}:1,2 -o ${_pot_file} ${_srcs}
        COMMAND ${GETTEXT_MSGMERGE_EXECUTABLE} -U --backup=none -q --lang=${locale} ${_po_file} ${_pot_file}
        COMMENT "Generating translation source file for ${proj_name}...")
    endif(TERRALIB_TRANSLATOR_ENABLED)
  endif(XGETTEXT_FOUND)
ENDMACRO(TERRALIB_CREATE_TRANSLATION)

